local _r = math.random
local _f = math.floor
local _ce = math.ceil
local _c = math.cos
local _s = math.sin
local _sq = math.sqrt
local _at2 = math.atan2
local _d2r = math.rad
local _r2d = math.deg
local _m = math.mod
local pi = math.pi
local gr = love.graphics
local ti = love.timer
local ms = love.mouse
local kb = love.keyboard
math.randomseed(os.time()) _r() _r() _r()

function love.run()
	love.load(arg)
	while true do
	  fps = ti.getFPS()
	  dt = ti.getDelta()
		ti.step()
		time = ti.getTime()
		gr.clear()
		love.update(dt)
		love.draw(dt)
		if love.event then
			for e,a,b,c in love.event.poll() do
			  if e == "kp" and player then player.idletime = 0 end
				if e == "q" then if love.audio then love.audio.stop() end return end
				love.handlers[e](a,b,c)
			end
		end
		ti.sleep(1)
		gr.present()
	end
end

function love.load()
	tiles = gr.newImage("tiles.png")

	tileQuad = {}
	for x=0,7 do
		tileQuad[x] = gr.newQuad(x * 16, 0 * 16, 16, 16, tiles:getWidth(), tiles:getHeight())
	end
  helpQuad = gr.newQuad(0, 16, 42, 28, tiles:getWidth(), tiles:getHeight())
  ladderQuad = {
    [1] = gr.newQuad(112, 16, 16, 16, tiles:getWidth(), tiles:getHeight()),
    [2] = gr.newQuad(96, 16, 16, 16, tiles:getWidth(), tiles:getHeight())
  }

	playerImg = gr.newImage("player.png")
	playerImg:setFilter("nearest", "nearest")

	playerQuad = {}
	for x=0,3 do
    playerQuad[x] = {}
    for y=0,3 do
  		playerQuad[x][y] = gr.newQuad(x * 8, y * 8, 8, 8, playerImg:getWidth(), playerImg:getHeight())
    end
	end

  playerZs = {}
	for x=0,2 do
    playerZs[x] = gr.newQuad((x+5) * 8, 0, 8, 8, playerImg:getWidth(), playerImg:getHeight())
  end

	playerGibs = {}
	for x=0,1 do
    playerGibs[x] = gr.newQuad(x * 4+32, 0, 4, 4, playerImg:getWidth(), playerImg:getHeight())
    playerGibs[x+2] = gr.newQuad(x * 4+32, 4, 4, 4, playerImg:getWidth(), playerImg:getHeight())
	end
  playerGibs[4] = gr.newQuad(32, 8, 16, 16, playerImg:getWidth(), playerImg:getHeight())
  playerGibs[5] = gr.newQuad(32, 8, 1, 1, playerImg:getWidth(), playerImg:getHeight())

  color = {}
  color[3] = {196,207,161}
  color[2] = {139,149,109}
  color[1] = {107,115,83}
  color[0] = {37,41,29}

	font = gr.newImageFont("font.png", "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 :-xtc!.,", 0)

  inMenu = 0
  inGame = 1
  inTran = 2

  gr.setFont(font)

  paused = false

  gamemode = inMenu
end

function love.update(dt)
  local kAction = kb.isDown(" ")
  local kLeft = kb.isDown("left") or kb.isDown("a")
  local kRight = kb.isDown("right") or kb.isDown("d")
  local kUp = kb.isDown("up") or kb.isDown("w")
  local kDown = kb.isDown("down") or kb.isDown("s")
  local kShift = kb.isDown("lshift") or kb.isDown("rshift")

  if gamemode == inTran then
    intro.done = 0
    for i=0,39 do
      intro.drip[i] = intro.drip[i] + intro.dripSpeed[i] * dt
      if intro.drip[i] >= 20 then intro.drip[i] = 20 end
      if intro.drip[i] == 20 then intro.done = intro.done + 1 end
    end
    if intro.done >= 40 then
      gamemode = inGame
    end

  elseif gamemode == inGame then
    if paused == false then
      player.idletime = player.idletime + dt

      if player.dying == false and level.finished == false then
        level.time = level.time + dt
        if level.time < 0 then
          level.time = 0
          killPlayer("      TIME UP!       ")
        end
      end

      if level.hit[_f(player.x)][_f(player.y)+1] == 0 then
        player.onfloor = false
      end

      if player.dying == false then
        if player.onladder == true then
          -- Let go of the ladder if you are on it, possibly falling to your doom!
          if kLeft and kRight == false then
            player.onladder = false
          elseif kRight and kLeft == false then
            player.onladder = false
          end

          -- Move up or down
          if kUp and kDown == false then
            if player.step_y == 0 then
              if level.ladder[player.x][player.y-1] > 0 then
                player.y = player.y - 1
                player.step_y = 8
              elseif level.hit[player.x][player.y+1] > 0 then
                player.onfloor = true
                player.onladder = false
              end
            end
          elseif kDown and kUp == false then
            if player.step_y == 0 then
              if level.ladder[player.x][player.y+1] > 0 then
                player.y = player.y + 1
                player.step_y = -8
              elseif level.hit[player.x][player.y+1] > 0 then
                player.onfloor = true
                player.onladder = false
              end
            end
          end

          if player.step_y > 0 then
            player.step_y = player.step_y - player.speed_y * dt
            if player.step_x < 0 then
              player.step_y = 0
              playerReachedTile()
              if level.hit[player.x][player.y+1] > 0 then
                player.onfloor = true
              end
            end
          elseif player.step_y < 0 then
            player.step_y = player.step_y + player.speed_y * dt
            if player.step_y > 0 then
              player.step_y = 0
              playerReachedTile()
              if level.hit[player.x][player.y+1] > 0 then
                player.onfloor = true
              end
            end
          end

          if player.step_y ~= 0 then
            player.step_f = player.step_f + 8 * dt
            if player.step_f >= 2 then player.step_f = 0 end
          end
        else
          if player.onfloor == false and player.step_x == 0 then
            player.step_f = 1
            player.step_y = player.step_y + player.speed_f * dt
            if player.step_y > 8 then
              player.y = player.y + 1
              player.step_y = 0
              player.falldistance = player.falldistance + 1
              if player.falldistance > 3 then
                player.rotate = player.rotate + 45
                if player.rotate > 180 then player.rotate = 180 end
              end
              if level.hit[_f(player.x)][_f(player.y)+1] == 1 then
                player.onfloor = true
                player.step_f = 0
                if player.falldistance > 3 then
                  killPlayer("       SPLAT!       ", true)
                elseif player.y == 16 then
                  killPlayer("      IMPALED!      ")
                else
                  player.falldistance = 0
                end
              end
            end
          end

          if player.onfloor == true and player.jumping == false and level.finished == false then
            if kLeft and kRight == false then
              if player.facing == 0 then
                player.facing = 1
                waitForRelease = .1
              elseif waitForRelease == 0 then
                if player.step_x == 0 and level.hit[player.x-1][player.y] == 0 then
                  if (level.hit[player.x-1][player.y] == 0 and level.hit[player.x-1][player.y+1] == 0 and
                      level.hit[player.x-2][player.y] == 0 and level.hit[player.x-2][player.y+1] == 1) then
                    player.jumping = true
                    player.jump_f = 16
                  else
                    player.x = player.x - 1
                    player.step_x = 8
                  end
                end
              end
            elseif kRight and kLeft == false then
              if player.facing == 1 then
                player.facing = 0
                waitForRelease = .1
              elseif waitForRelease == 0 then
                if player.step_x == 0 and level.hit[player.x+1][player.y] == 0 then
                  if (player.facing == 0 and level.hit[player.x+1][player.y] == 0 and level.hit[player.x+1][player.y+1] == 0 and
                      level.hit[player.x+2][player.y] == 0 and level.hit[player.x+2][player.y+1] == 1) then
                    player.jumping = true
                    player.jump_f = 16
                  else
                    player.x = player.x + 1
                    player.step_x = -8
                  end
                end
              end
            end
          end
--[[        OLD JUMP CODE NOT NEEDED SINCE AUTO-JUMP UPDATE // REMOVE/ARCHIVE LATER
            if kAction and player.jumping == false and player.step_x == 0 and player.onfloor == true then
            if (player.facing == 0 and level.hit[player.x+1][player.y] == 0 and level.hit[player.x+1][player.y+1] == 0 and level.hit[player.x+2][player.y] == 0) or
              (player.facing == 1 and level.hit[player.x-1][player.y] == 0 and level.hit[player.x-1][player.y+1] == 0 and level.hit[player.x-2][player.y] == 0) then
              player.jumping = true
              player.jump_f = 16
            end
          end--]]

          if player.step_x == 0 then
            if kUp and kDown == false then
              if level.ladder[player.x][player.y] == 1 then
                player.onladder = true
                player.falldistance = 0
                player.rotate = 0
              end
            elseif kDown and kUp == false then
              if level.ladder[player.x][player.y+1] == 1 then
                player.onladder = true
                player.falldistance = 0
                player.rotate = 0
              end
            end
          end

          waitForRelease = waitForRelease - dt
          if waitForRelease < 0 then
            waitForRelease = 0
          end

          if player.jumping then
            player.step_f = 1
            player.step_y = -1
            player.jump_f = player.jump_f - player.speed_j * dt
            if player.facing == 0 then
              player.step_x = 16-player.jump_f
            elseif player.facing == 1 then
              player.step_x = player.jump_f-16
            end
            if player.jump_f < 0 then
              player.jump_f = 0
              player.jumping = false
              if player.facing == 0 then
                player.x = player.x + 2
              else
                player.x = player.x - 2
              end
              player.step_x = 0
              player.step_y = 0
              player.step_f = 0
              playerReachedTile()
            end
          else
            if player.step_x > 0 then
              player.step_x = player.step_x - player.speed_x * dt
              if player.step_x < 0 then player.step_x = 0 playerReachedTile() end
            elseif player.step_x < 0 then
              player.step_x = player.step_x + player.speed_x * dt
              if player.step_x > 0 then player.step_x = 0 playerReachedTile() end
            end
          end


          if player.step_x ~= 0 then
            player.step_f = player.step_f + 8 * dt
            if player.step_f >= 2 then player.step_f = 0 end
          end
        end

      else --END If Player NOT Dying, BEGIN Player IS Dying
        player.dying_f = player.dying_f - dt      --Dying frame advance
        player.rotate = player.rotate + 720 * dt  --Rotate player twice per second
        if player.dying_f < 0 then                --If the death animation is over...
          player.dying_f = 0                      --Reset frame to 0
          player.dying = false                    --Turn off death
          player.x = 2                            --Reset player position
          player.y = 0
          player.rotate = 0                       --Reset rotation
          player.jumping = false                  --In case he was jumping
          player.facing = 0                       --Reset facing
          player.idletime = 0                     --Reset idle time too
          player.gibs = false
          if player.lives <= 0 then               --Make sure player isn't out of lives
            gamemode = inMenu                     --If so, return to menu
          end
        end
      end

      if level.exit.animating == true then
        level.exit.frametime = level.exit.frametime - dt
        if level.exit.frametime < 0 then
          level.exit.frametime = .05
          level.exit.frame = level.exit.frame + 1
          if level.exit.frame > 20 then
            gamemode = inTran
            player.level = player.level + 1
            resetIntro()
            loadLevel(player.level)
          end
        end
      end
    end
  end
end

function love.draw()
  if gamemode == inMenu then
    gr.setBackgroundColor(unpack(color[3]))
    gr.setColor(unpack(color[0]))
    gr.print("\n\n\n\n\n\n\n\n\n\n\n\n    PRESS  SPACE\n\n      TO  PLAY\n\n\n    c2010 JASOCO", 0, -16)
  elseif gamemode == inTran then
    gr.setBackgroundColor(unpack(color[3]))
    gr.setColor(255,255,255)
    drawLevel()
    for i=0,39 do
      gr.setColor(unpack(color[0]))
      gr.rectangle("fill", i*8-2, intro.drip[i]*16-2, 12, 400)
    end
    for i=0,39 do
      gr.setColor(unpack(color[3]))
      gr.rectangle("fill", i*8, intro.drip[i]*16, 8, 400)
    end
  elseif gamemode == inGame then
    gr.setBackgroundColor(unpack(color[3]))
    gr.setColor(255,255,255)

    drawLevel()

    local psx, psy = 0, 0
    if level.finished then
      psx = 3
      psy = 1
      local _ = _c(time*20)
      if _ > 0 then psy = 0 end
    elseif player.onladder then
      psx = 2
      psy = _f(player.step_f)
    elseif player.onfloor == false and player.falldistance > 0 then
      psx = player.facing
      psy = 1
    elseif player.dying then
      psx = 2
      psy = 2
    elseif player.idletime > 3 then
      if player.idletime > 6 then
        psx = 3
        psy = 3
        zz = _f((player.idletime-6)*2%3)
        gr.drawq(playerImg, playerZs[zz], _f(player.x*8)*2+8+8, _f(player.y*8)*2+8-16, 0, 2, 2, 4, 4)
      elseif player.idletime > 4.5 then
        psx = 3
        psy = 3
      elseif player.idletime > 4 then
        psx = 3
        psy = 2
      else
        psx = 3
        psy = 0
      end
    else
      psx = player.facing
      psy = _f(player.step_f)
    end
    if player.gibs == false then
      gr.drawq(playerImg, playerQuad[psx][psy], _f(player.x*8 + player.step_x)*2+8, _f(player.y*8 + player.step_y)*2+8, _d2r(_f(player.rotate/90)*90), 2, 2, 4, 4)
    end

    for i, t in pairs(trap) do
      local st = 0
      if t.active then
        if paused == false then
          t.shaketime = t.shaketime - dt
          if t.shaketime < 0 then
            t.shaketime = 0
            t.y = t.y + t.speed * dt
          else
            st = _f(_c(t.shaketime*100)) * 2
          end
          if _f(t.x/16) == player.x and _f(t.y/16) == player.y and t.alive then
            local tt
            if t.type == 1 then tt = "    THAT SMARTS!    " end
            killPlayer(tt, true)
            t.active = false
            t.alive = false
          end
        end
      end
      if t.alive == true then gr.drawq(t.img, t.quad, t.x+st, t.y) end
    end

    local helpTick = _c(level.time*4)
    if helpTick > 0 and level.finished == false then
      gr.drawq(tiles, helpQuad, (level.exit.x*16) - 30, (level.exit.y*16) - 30)
    end

    drawStatusBar()

    drawDeathScreen()
    drawPauseScreen()
  end

  gr.setColor(unpack(color[3]))
  gr.print(fps, 18*16, -16)
end

function love.mousepressed(x, y, button)
  if button == "l" then
    --DEBUG:
    print(_f(x/16),_f(y/16))
  end
end

function love.keypressed(k)
  if k == " " then
    if gamemode == inMenu then
      gamemode = inTran
      setupGame()
    end
  elseif k == "x" then
    local x, y = _r(1,18), 0
    if level.hit[x][y] == 1 then createTrap(1, x*16, y, 16, 16, 128, tiles, tileQuad[level.tile[x][y]]) end
    level.tile[x][y] = 3
    level.hit[x][y] = 0
  elseif k == "p" then
    paused = not paused
  end
end

function killPlayer(dr, gib)
  if player.dying == false then
    player.deathreason = dr
    player.lives = player.lives - 1
    player.dying = true
    player.dying_f = 3
    player.falldistance = 0
    player.gibs = gib
    if gib then
      for i=0,11 do
        player.gib_pos.x[i] = (player.x*8+4)*2
        player.gib_pos.y[i] = (player.y*8+4)*2
      end
      player.gib_pos.xs[0] = -16
      player.gib_pos.xs[1] = 16
      player.gib_pos.xs[2] = -32
      player.gib_pos.xs[3] = 32
      player.gib_pos.ys[0] = -64
      player.gib_pos.ys[1] = -64
      player.gib_pos.ys[2] = -32
      player.gib_pos.ys[3] = -32
      for i=4,11 do
        player.gib_pos.xs[i] = _r(-32,32)
        player.gib_pos.ys[i] = _r(-64,64)
      end
    end
    print("Kill Player")
  end
end

function drawDeathScreen()
  if player.dying then
    if player.gibs then
      gr.setColor(255,255,255)
      for i=0,11 do
        player.gib_pos.x[i] = player.gib_pos.x[i] + player.gib_pos.xs[i] * dt
        player.gib_pos.y[i] = player.gib_pos.y[i] + player.gib_pos.ys[i] * dt
        player.gib_pos.ys[i] = player.gib_pos.ys[i] + 128 * dt

        local xc, yc = _f(player.gib_pos.x[i]/16), _f(player.gib_pos.y[i]/16)
        if xc < 0 then xc = 0 end
        if yc < 0 then yc = 0 end
        if xc > 19 then xc = 19 end
        if yc > 19 then yc = 19 end
        if level.hit[xc][yc] > 0 then
          player.gib_pos.ys[i] = -(player.gib_pos.ys[i]/2)
          player.gib_pos.xs[i] = player.gib_pos.xs[i] * .8
        end

        if i < 4 then
          gr.drawq(playerImg, playerGibs[i], player.gib_pos.x[i], player.gib_pos.y[i], 0, 2, 2, 2, 3)
        else
          gr.drawq(playerImg, playerGibs[5], player.gib_pos.x[i], player.gib_pos.y[i], 0, 2, 2, 0, 1)
        end
      end
      if player.dying_f > 2.95 then
        gr.drawq(playerImg, playerGibs[4], player.x * 16+8, player.y * 16+8, 0, 2, 2, 4, 4)
      end
    end

    local t = (player.y+3)*16
    if player.y >= 13 then t = (player.y-4)*16 end
    gr.setColor(unpack(color[2]))
    gr.rectangle("fill", 0, t+6, 320, 36)
    gr.setColor(unpack(color[3]))
    gr.rectangle("fill", 0, t+8, 320, 32)
    gr.setColor(unpack(color[2]))
    gr.print(player.deathreason, 0, t+2)
    gr.setColor(unpack(color[0]))
    gr.print(player.deathreason, 0, t)
  end
end

function drawPauseScreen()
  if paused then
    local t = 7*16
    gr.setColor(unpack(color[2]))
    gr.rectangle("fill", 0, t+6, 320, 36)
    gr.setColor(unpack(color[3]))
    gr.rectangle("fill", 0, t+8, 320, 32)
    gr.setColor(unpack(color[2]))
    gr.print("       PAUSED        ", 0, t+2)
    gr.setColor(unpack(color[0]))
    gr.print("       PAUSED        ", 0, t)
  end
end

function setupGame()
  resetIntro()

  enemy = {}
  trap = {}

	player = {
	  x, y, step_x = 0, step_y = 0, step_f = 0, speed_x = 32, speed_y = 32, speed_f = 64, speed_j = 32,
	  facing = 0, lives = 3, onfloor = false, falldistance = 0, rotate = 0, idletime = 0, level = 1,
	  dying = false, dying_f = 0, deathreason = "", gibs = false, gib_pos = { x = {}, y = {}, xs = {}, ys = {} },
	  jumping = false, jump_f = 0, onladder = false
	}

  loadLevel(1)

	waitForRelease = 0
end

function createTrap(t, x, y, w, h, s, img, quad)
  local n = #trap+1
  if t == 1 then
    trap[n] = {
      x = x, y = y, w = w, h = h, speed = s,
      shaketime = 0, hasshake = true, type = t, active = false, alive = true,
      img = img, quad = quad
    }
  end
end

function resetIntro()
  intro = { drip = {}, dripSpeed = {}, done = 0 }
	for x=0,39 do
    intro.drip[x] = -2
    intro.dripSpeed[x] = _r(20,40)
    if x > 0 and intro.dripSpeed[x] == intro.dripSpeed[x-1] then intro.dripSpeed[x] = intro.dripSpeed[x] + 1 end
	end
end

function loadLevel(l)
  enemy = {}
  trap = {}

  level = {
    hit = {}, tile = {}, ladder = {}, time = 0, start = ti.getTime(), finished = false,
    exit = { x = 16, y = 15, animating = false, frame = 1, frametime = 0 }
  }
	for x=0,19 do
	  level.hit[x] = {}
	  level.tile[x] = {}
	  level.ladder[x] = {}
  	for y=0,17 do
  	  level.hit[x][y] = 1
  	  level.tile[x][y] = 0
  	  level.ladder[x][y] = 0
    end
	end

	for x=1,18 do
  	for y=1,15 do
  	  level.hit[x][y] = 0
  	  level.tile[x][y] = 3
    end
	end

	for i=1,20 do
  	level.tile[_r(1,17)][_r(1,14)] = 2
	end

	level.tile[level.exit.x][level.exit.y] = 5
	level.tile[2][0] = 3
	level.hit[2][0] = 0

	for i=1,2 do
	  level.hit[i][3] = 1
	  level.tile[i][3] = 1
	end

	for i=1,4 do
	  level.hit[i][5] = 1
	  level.tile[i][5] = 1
	end
	for i=3,6 do
	  level.hit[i][8] = 1
	  level.tile[i][8] = 1
	end
	for i=8,11 do
	  level.hit[i][8] = 1
	  level.tile[i][8] = 1
	end
	for i=11,13 do
	  level.hit[i][11] = 1
	  level.tile[i][11] = 1
	end

	for i=10,11 do
	  level.hit[i][13] = 1
	  level.tile[i][13] = 1
	end

	for i=8,15 do
	  level.ladder[4][i] = 1
	end

  level.hit[1][16] = 0
  level.tile[1][16] = 4
  level.hit[6][16] = 0
  level.tile[6][16] = 4
  level.hit[5][16] = 0
  level.tile[5][16] = 4

	for x=1,18 do
  	for y=1,15 do
  	  if level.ladder[x][y] == 1 and level.ladder[x][y-1] == 0 then
  	    level.ladder[x][y-1] = 2
  	  end
    end
	end


  player.x = 2
  player.y = 0

  createTrap(1, 10*16, 1*16, 16, 16, 128, tiles, tileQuad[7])
  createTrap(1, 9*16, 1*16, 16, 16, 128, tiles, tileQuad[7])

  createTrap(1, 9*16, 8.5*16, 16, 16, 128, tiles, tileQuad[7])
end

function playerReachedTile()
  player.idletime = 0
  player.step_f = 0

  -- Check for hole immediately to left or right
  local xx
  if player.facing == 0 then xx = player.x+1 else xx = player.x-1 end
  if level.hit[xx][player.y+1] == 0 then
    waitForRelease = .1
  end
  if player.x == level.exit.x and player.y == level.exit.y then
    level.exit.animating = true
    level.exit.frame = 0
    level.exit.frametime = .1
    level.finished = true
  end

  -- Check for trap to be activated
  for i, t in pairs(trap) do
    if player.x >= _f(t.x/16)-1 and player.x <= _f(t.x/16)+1 and player.y >= _f(t.y/16) and t.active == false then
      t.active = true
      if t.hasshake then t.shaketime = .3 end
    end
  end
end

function drawStatusBar()
  gr.setColor(unpack(color[3]))
  gr.rectangle("fill", 0, 288-16, 320, 16)
  gr.setColor(unpack(color[0]))
  gr.print("  x"..player.lives.."  LV:"..player.level.."  t"..formatTime(level.time, 2), 0, 8*16*2)
  gr.setColor(255,255,255)
  gr.drawq(playerImg, playerQuad[3][1], 1*16, 17*16, 0, 2, 2)
end

function drawLevel()
  for x=0,19 do
    for y=0,16 do
      gr.drawq(tiles, tileQuad[level.tile[x][y]], (x*8)*2, (y*8)*2)
      if level.ladder[x][y] > 0 then
        gr.drawq(tiles, ladderQuad[level.ladder[x][y]], x*16, y*16)
      end
    end
  end
end

function degToRad(d) return d * pi / 180 end

function sort(T) table.sort(T, function(a, b) return a.y < b.y end ) end

function formatNumber(number) return (string.format("%d", number):reverse():gsub( "(%d%d%d)" , "%1," ):reverse():gsub("^(-?),","%1")) end

--GENERAL MATH RELATED FUNCTIONS
function distanceFrom(x1,y1,x2,y2) return _sq((x2 - x1) ^ 2 + (y2 - y1) ^ 2) end

function findAngle(x1, y1, x2, y2)
	local t = _r2d(_at2(y1 - y2, x2 - x1))
	if t < 0 then t = t + 360 end
	return t
end

function formatTime(t, p, ms)
  local h = _f(_m(t/60/60,60))
  local m = string.sub("0" .. _f(_m(t/60,60)),-2)
  local s = string.sub("0" .. _f(_m(t,60)),-2)
  local r = ""
  if p then
    if h > 0 then
      r = h .. ":" .. m
    else
      r = m .. ":" .. s
    end
  else
    r = h .. ":" .. m .. ":" .. s
  end
  if ms then r = r .. (t - _f(t)) end
  return r
end

function trim(s) return (string.gsub(s, "^%s*(.-)%s*$", "%1")) end

function twoDown(n) return _f((n) / 2) * 2 end
